@@ -2,7 +2,9 @@
 
 require 'log4r'
 
+require 'vagrant-libvirt/util/erb_template'
 require 'vagrant-libvirt/util/resolvers'
+require 'vagrant-libvirt/util/storage_util'
 
 module VagrantPlugins
   module ProviderLibvirt
@@ -44,6 +46,7 @@ module VagrantPlugins
           @nvram = config.nvram
           @machine_type = config.machine_type
           @machine_arch = config.machine_arch
+          @disk_controller_model = config.disk_controller_model
           @disk_driver_opts = config.disk_driver_opts
           @nested = config.nested
           @memory_size = config.memory.to_i * 1024
@@ -73,6 +76,22 @@ module VagrantPlugins
           @tpm_path = config.tpm_path
           @tpm_version = config.tpm_version
 
+          @sysinfo = config.sysinfo.dup
+          @sysinfo.each do |section, _v|
+            if @sysinfo[section].respond_to?(:each_pair)
+              @sysinfo[section].delete_if { |_k, v| v.to_s.empty? }
+            else
+              @sysinfo[section].reject! { |e| e.to_s.empty? }
+            end
+          end.reject! { |_k, v| v.empty? }
+          @sysinfo_blocks = {
+            'bios' => {:ui => "BIOS", :xml => "bios"},
+            'system' => {:ui => "System", :xml => "system"},
+            'base board' => {:ui => "Base Board", :xml => "baseBoard"},
+            'chassis' => {:ui => "Chassis", :xml => "chassis"},
+            'oem strings' => {:ui => "OEM Strings", :xml => "oemStrings"},
+          }
+
           # Boot order
           @boot_order = config.boot_order
 
@@ -151,9 +170,11 @@ module VagrantPlugins
             rescue Libvirt::Error => e
               # It is hard to believe that e contains just a string
               # and no useful error code!
-              msg = "Call to virStorageVolCreateXML failed: " +
-                    "storage volume '#{disk[:absolute_path]}' exists already"
-              if e.message == msg and disk[:allow_existing]
+              msgs = [disk[:name], disk[:absolute_path]].map do |name|
+                "Call to virStorageVolCreateXML failed: " +
+                "storage volume '#{name}' exists already"
+              end
+              if msgs.include?(e.message) and disk[:allow_existing]
                 disk[:preexisting] = true
               else
                 raise Errors::FogCreateDomainVolumeError,
@@ -243,7 +264,7 @@ module VagrantPlugins
           env[:ui].info(" -- Video Type:        #{@video_type}")
           env[:ui].info(" -- Video VRAM:        #{@video_vram}")
           env[:ui].info(" -- Video 3D accel:    #{@video_accel3d}")
-          env[:ui].info(" -- Sound Type:	#{@sound_type}")
+          env[:ui].info(" -- Sound Type:        #{@sound_type}")
           env[:ui].info(" -- Keymap:            #{@keymap}")
           env[:ui].info(" -- TPM Backend:       #{@tpm_type}")
           if @tpm_type == 'emulator'
@@ -253,6 +274,22 @@ module VagrantPlugins
             env[:ui].info(" -- TPM Path:          #{@tpm_path}")
           end
 
+          unless @sysinfo.empty?
+            env[:ui].info(" -- Sysinfo:")
+            @sysinfo.each_pair do |block, values|
+              env[:ui].info("   -- #{@sysinfo_blocks[block.to_s][:ui]}:")
+              if values.respond_to?(:each_pair)
+                values.each_pair do |name, value|
+                  env[:ui].info("    -> #{name}: #{value}")
+                end
+              else
+                values.each do |value|
+                  env[:ui].info("    -> #{value}")
+                end
+              end
+            end
+          end
+
           if @memballoon_enabled
             env[:ui].info(" -- Memballoon model:  #{@memballoon_model}")
             env[:ui].info(" -- Memballoon bus:    #{@memballoon_pci_bus}")
