@@ -2,11 +2,10 @@
 
 require 'log4r'
 
-begin
-  require 'rexml'
-rescue LoadError
-  require 'rexml/rexml'
-end
+require 'rexml/document'
+require 'rexml/xpath'
+
+require 'vagrant-libvirt/util/domain_flags'
 
 module VagrantPlugins
   module ProviderLibvirt
@@ -34,7 +33,7 @@ module VagrantPlugins
               begin
                 libvirt_domain.lookup_snapshot_by_name(name).delete
               rescue => e
-                raise Errors::DeleteSnapshotError, error_message: e.message
+                raise Errors::SnapshotDeletionError, error_message: e.message
               end
             end
           rescue
