@@ -152,6 +152,88 @@ describe VagrantPlugins::ProviderLibvirt::Action::CreateDomain do
           expect(subject.call(env)).to be_nil
         end
       end
+
+      context 'with disk controller model virtio-scsi' do
+        before do
+          allow(machine.provider_config).to receive(:disk_controller_model).and_return('virtio-scsi')
+          expect(volumes).to receive(:all).with(name: 'vagrant-test_default.img').and_return([domain_volume])
+
+          env[:domain_volumes][0][:bus] = 'scsi'
+        end
+
+        it 'should add a virtio-scsi disk controller' do
+          expect(ui).to receive(:info).with(/ -- Image\(vda\):.*/)
+          expect(servers).to receive(:create) do |args|
+            expect(args[:xml]).to match(/<controller type='scsi' model='virtio-scsi' index='0'\/>/)
+          end.and_return(machine)
+
+          expect(subject.call(env)).to be_nil
+        end
+      end
+
+      context 'sysinfo' do
+        let(:domain_xml_file) { 'sysinfo.xml' }
+        let(:vagrantfile_providerconfig) do
+          <<-EOF
+          libvirt.sysinfo = {
+            'bios': {
+              'vendor': 'Test Vendor',
+              'version': '',
+            },
+            'system': {
+              'manufacturer': 'Test Manufacturer',
+              'version': '0.1.0',
+              'serial': '',
+            },
+            'base board': {
+              'manufacturer': 'Test Manufacturer',
+              'version': '',
+            },
+            'chassis': {
+              'manufacturer': 'Test Manufacturer',
+              'serial': 'AABBCCDDEE',
+              'asset': '',
+            },
+            'oem strings': [
+              'app1: string1',
+              'app1: string2',
+              'app2: string1',
+              'app2: string2',
+              '',
+              '',
+            ],
+          }
+          EOF
+        end
+
+        it 'should populate sysinfo as expected' do
+          expect(servers).to receive(:create).with(xml: domain_xml).and_return(machine)
+
+          expect(subject.call(env)).to be_nil
+        end
+
+        context 'with block of empty entries' do
+          let(:domain_xml_file) { 'sysinfo_only_required.xml' }
+          let(:vagrantfile_providerconfig) do
+            <<-EOF
+            libvirt.sysinfo = {
+              'bios': {
+                'vendor': 'Test Vendor',
+              },
+              'system': {
+                'serial': '',
+              },
+            }
+            EOF
+          end
+
+          it 'should skip outputting the surrounding tags' do
+            expect(servers).to receive(:create).with(xml: domain_xml).and_return(machine)
+
+            expect(subject.call(env)).to be_nil
+          end
+        end
+      end
     end
 
     context 'connection => qemu:///session' do
