@@ -18,12 +18,12 @@ describe VagrantPlugins::ProviderLibvirt::Action::WaitTillUp do
 
   describe '#call' do
     before do
-      allow_any_instance_of(VagrantPlugins::ProviderLibvirt::Provider).to receive(:driver)
-        .and_return(driver)
       allow(driver).to receive(:get_domain).and_return(domain)
       allow(driver).to receive(:state).and_return(:running)
       # return some information for domain when needed
       allow(domain).to receive(:mac).and_return('9C:D5:53:F1:5A:E7')
+
+      allow(machine.provider_config).to receive(:qemu_use_session).and_return(false)
     end
 
     context 'when machine does not exist' do
@@ -86,46 +86,4 @@ describe VagrantPlugins::ProviderLibvirt::Action::WaitTillUp do
       end
     end
   end
-
-  describe '#recover' do
-    before do
-      allow_any_instance_of(VagrantPlugins::ProviderLibvirt::Driver).to receive(:get_domain).and_return(machine)
-      allow_any_instance_of(VagrantPlugins::ProviderLibvirt::Driver).to receive(:state)
-        .and_return(:not_created)
-      allow(env).to receive(:[]).and_call_original
-    end
-
-    it 'should do nothing by default' do
-      expect(env).to_not receive(:[]).with(:action_runner) # cleanup
-      expect(subject.recover(env)).to be_nil
-    end
-
-    context 'with machine coming up' do
-      before do
-        allow_any_instance_of(VagrantPlugins::ProviderLibvirt::Driver).to receive(:state)
-          .and_return(:running)
-        env[:destroy_on_error] = true
-      end
-
-      context 'and user has disabled destroy on failure' do
-        before do
-          env[:destroy_on_error] = false
-        end
-
-        it 'skips terminate on failure' do
-          expect(env).to_not receive(:[]).with(:action_runner) # cleanup
-          expect(subject.recover(env)).to be_nil
-        end
-      end
-
-      context 'and using default settings' do
-        let(:runner) { double('runner') }
-        it 'deletes VM on failure' do
-          expect(env).to receive(:[]).with(:action_runner).and_return(runner) # cleanup
-          expect(runner).to receive(:run)
-          expect(subject.recover(env)).to be_nil
-        end
-      end
-    end
-  end
 end
