@@ -197,6 +197,13 @@ describe VagrantPlugins::ProviderLibvirt::Config do
             :env => {'LIBVIRT_DEFAULT_URI' => 'qemu:///custom'},
           }
         ],
+        [ # when username explicitly set with @ symbol for domains
+          {:username => 'my_user@my_domain', :host => 'remote'},
+          {:uri => %r{qemu://remote/(system|session)}, :username => 'my_user@my_domain'},
+          {
+            :env => {'LIBVIRT_DEFAULT_URI' => 'qemu:///custom'},
+          }
+        ],
         [ # when username explicitly set with host but without ssh
           {:username => 'my_user', :host => 'remote'},
           {:uri => %r{qemu://remote/(system|session)}, :username => 'my_user'},
@@ -577,24 +584,28 @@ describe VagrantPlugins::ProviderLibvirt::Config do
 
   def assert_invalid
     subject.finalize!
-    errors = subject.validate(machine)
-    raise "No errors: #{errors.inspect}" if errors.values.all?(&:empty?)
+    errors = subject.validate(machine).values.first
+    expect(errors).to_not be_empty
+    errors
   end
 
   def assert_valid
     subject.finalize!
-    errors = subject.validate(machine)
-    raise "Errors: #{errors.inspect}" unless errors.values.all?(&:empty?)
+    errors = subject.validate(machine).values.first
+    expect(errors).to be_empty
   end
 
   describe '#validate' do
+    before do
+      allow(machine).to receive(:provider_config).and_return(subject)
+      allow(machine).to receive(:ui).and_return(ui)
+    end
+
     it 'is valid with defaults' do
       assert_valid
     end
 
     context 'with disks defined' do
-      before { expect(machine).to receive(:provider_config).and_return(subject).at_least(:once) }
-
       it 'is valid if relative path used for disk' do
         subject.storage :file, path: '../path/to/file.qcow2'
         assert_valid
@@ -630,6 +641,63 @@ describe VagrantPlugins::ProviderLibvirt::Config do
       end
     end
 
+    context 'with public_network defined' do
+      let(:libvirt_client)  { instance_double(::Libvirt::Connect) }
+      let(:host_devices) { [
+        instance_double(Libvirt::Interface),
+        instance_double(Libvirt::Interface),
+      ] }
+      let(:libvirt_networks) { [
+        instance_double(Libvirt::Network),
+        instance_double(Libvirt::Network),
+      ] }
+      let(:driver) { instance_double(::VagrantPlugins::ProviderLibvirt::Driver) }
+      before do
+        machine.config.vm.network :public_network, dev: 'eth0', ip: "192.168.2.157"
+        allow(machine.provider).to receive(:driver).and_return(driver)
+        expect(driver).to receive(:list_host_devices).and_return(host_devices)
+        expect(driver).to receive(:list_networks).and_return(libvirt_networks)
+        expect(host_devices[0]).to receive(:name).and_return('eth0')
+        expect(host_devices[1]).to receive(:name).and_return('virbr0')
+        expect(libvirt_networks[0]).to receive(:bridge_name).and_return('')
+        expect(libvirt_networks[1]).to receive(:bridge_name).and_return('virbr0')
+      end
+
+      it 'should validate use of existing device' do
+        assert_valid
+      end
+
+      context 'when default device not on host' do
+        before do
+          machine.config.vm.network :public_network, dev: 'eno1', ip: "192.168.2.157"
+        end
+
+        it 'should be invalid' do
+          assert_invalid
+        end
+      end
+
+      context 'when using excluded host device' do
+        before do
+          machine.config.vm.network :public_network, dev: 'virbr0', ip: "192.168.2.157"
+        end
+
+        it 'should be invalid' do
+          assert_invalid
+        end
+
+        context 'when user overrides to allow device' do
+          before do
+            subject.host_device_exclude_prefixes = []
+          end
+
+          it 'should validate' do
+            assert_valid
+          end
+        end
+      end
+    end
+
     context 'with nvram defined' do
       before do
         subject.nvram = '/path/to/some/nvram'
@@ -662,6 +730,60 @@ describe VagrantPlugins::ProviderLibvirt::Config do
         assert_valid
       end
     end
+
+    context 'with sysinfo defined' do
+      context 'when invalid block name provided' do
+        it 'should be invalid' do
+          subject.sysinfo = {'bad bios': {'vendor': 'some vendor'}}
+
+          errors = assert_invalid
+          expect(errors).to include(match(/invalid sysinfo element 'bad bios';/))
+        end
+      end
+
+      context 'when invalid element name provided' do
+        it 'should be invalid' do
+          subject.sysinfo = {'bios': {'bad vendor': 'some vendor'}}
+
+          errors = assert_invalid
+          expect(errors).to include(match(/'sysinfo.bios' does not support entry name 'bad vendor'/))
+        end
+      end
+
+      context 'when empty element value provided' do
+        it 'should succeed with a warning' do
+          expect(ui).to receive(:warn).with(/Libvirt Provider: sysinfo.bios.vendor is nil or empty/)
+          subject.sysinfo = {'bios': {'vendor': ''}}
+
+          assert_valid
+        end
+      end
+
+      context 'when handling "oem strings"' do
+        it 'should succeed' do
+          subject.sysinfo = {'oem strings': ['string 1']}
+
+          assert_valid
+        end
+
+        context 'when empty entries' do
+          it 'should succeed with a warning' do
+            expect(ui).to receive(:warn).with(/Libvirt Provider: 'sysinfo.oem strings' contains an empty/)
+            subject.sysinfo = {'oem strings': ['']}
+
+            assert_valid
+          end
+        end
+
+        context 'when non string passed' do
+          it 'should be invalid' do
+            subject.sysinfo = {'oem strings': [true]}
+
+            assert_invalid
+          end
+        end
+      end
+    end
   end
 
   describe '#merge' do
@@ -725,5 +847,45 @@ describe VagrantPlugins::ProviderLibvirt::Config do
                                                 include(name: 'hpet'))
       end
     end
+
+    context 'sysinfo' do
+      it 'should merge' do
+        one.sysinfo = {
+          'bios' => {'vendor': 'Some Vendor'},
+          'system' => {'manufacturer': 'some manufacturer'},
+          'oem strings' => ['string 1'],
+        }
+        two.sysinfo = {
+          'bios' => {'vendor': 'Another Vendor'},
+          'system' => {'serial': 'AABBCCDDEE'},
+          'oem strings' => ['string 2'],
+        }
+
+        subject.finalize!
+        expect(subject.sysinfo).to eq(
+          'bios' => {'vendor': 'Another Vendor'},
+          'system' => {'manufacturer': 'some manufacturer', 'serial': 'AABBCCDDEE'},
+          'oem strings' => ['string 1', 'string 2'],
+        )
+      end
+    end
+
+    context 'boot_order' do
+      it 'should merge' do
+        one.boot 'network'
+
+        subject.finalize!
+        expect(subject.boot_order).to eq(['network'])
+      end
+
+      it 'should have last definition win' do
+        one.boot 'network'
+        two.boot 'hd'
+        two.boot 'cdrom'
+
+        subject.finalize!
+        expect(subject.boot_order).to eq(['hd', 'cdrom'])
+      end
+    end
   end
 end
